/*
 * @Author: Kane
 * @Date: 2023-10-08 15:02:15
 * @LastEditors: Kane
 * @FilePath: /desktop_archievement_backend/src/main/java/com/cpic/xim/utils/data/ImportBIExcelData.java
 * @Description: 坐席业绩相关的数据操作方法。
 * 
 * Copyright (c) ${2023} by Kane, All Rights Reserved. 
 */
package com.cpic.xim.utils.data;

import java.io.File;
import java.io.IOException;
import java.time.LocalDate;
import java.util.ArrayList;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.apache.poi.openxml4j.exceptions.InvalidFormatException;
import org.apache.poi.ss.usermodel.Row;
import org.apache.poi.ss.usermodel.Sheet;
import org.apache.poi.ss.usermodel.Workbook;
import org.apache.poi.ss.usermodel.WorkbookFactory;
import com.cpic.xim.mybatis.pojo.*;
import com.cpic.xim.utils.poi.MyPOIUtils;

/**
 * 坐席业绩相关的数据操作方法。
 */
public final class ImportBIExcelData
{
    private static Logger   logger                           =
            LoggerFactory.getLogger( ImportBIExcelData.class );

    private static String[] TelsalerAttachingRateExcelTitle  = new String[]
    { "部门", "经办", "车险保费(万)", "车险保费占比", "非车保费(万)", "当月保费渗透率", "保费渗透率环比上月", "当月客户渗透率", "客户渗透率环比上月",
            "当月车非客均保费", "客均保费环比上月"};

    private static String[] TelSalerRenewalRateExcelTitle    = new String[]
    { "责任人", "机构目标值1(%)", "到期数-全月", "序时到期数占比(%)", "个车续保率（序时）(%)", "个车续保率（全月）(%)", "环比昨日(%)",
            "环比上月(%)"};

    private static String[] DepartmentArchievementExcelTitle = new String[]
    { "部门", "目标值-机构", "目标差距", "车险保费(万)", "车险保费占比", "非车保费(万)", "当月保费渗透率", "保费渗透率环比上月", "当月客户渗透率",
            "客户渗透率环比上月", "当月车非客均保费", "客均保费环比上月"};

    /**
     * 从excel文件读取坐席的车非渗透率数据
     * @param filePath 文件路径
     * @param summaryDate 统计日期
     * @return 返回一个ArrayList保存的TelsalerAttachingRateRecord记录数组。
     * @throws IOException 打开excel文件错误时抛出。
     * @throws InvalidFormatException excel单元格格式错误时抛出
     */
    public static ArrayList<BITelsalerAttachingRateRecord> importBITelsalerAttachingRateRecordFromXlsx(
            String filePath, int SheetIndex, boolean hasCaptionRow )
            throws IOException, InvalidFormatException
    {
        ArrayList<BITelsalerAttachingRateRecord> records = new ArrayList<>( 200 );

        Workbook wb = null;
        Sheet sheet = null;

        try
        {
            wb = WorkbookFactory.create( new File( filePath ) );
            sheet = wb.getSheetAt( SheetIndex );

            for ( Row row : sheet )
            {
                String name = "";
                int rowIndex = row.getRowNum();

                // 判断是否要跳过标题行
                if ( hasCaptionRow == true && row.getRowNum() == 0 )
                {
                    continue;
                }

                try
                {
                    // 坐席名称
                    name = MyPOIUtils.getStringCellValue( row, 1 );

                    // 忽略坐席名称为空的行
                    if ( name.equals( "全流程" ) || name.equals( "其他" ) || name.isEmpty() )
                    {
                        continue;
                    }

                    // 车险保费
                    double motoPremium = MyPOIUtils.getNumbericCellValue( row, 2 );
                    // 非车险保费
                    double nomotoPremium = MyPOIUtils.getNumbericCellValue( row, 4 );
                    // 车险保费占比
                    double motoPremiumProPortion = MyPOIUtils.getNumbericCellValue( row, 3 );
                    // 当月保费渗透率
                    double attachingRate = MyPOIUtils.getNumbericCellValue( row, 5 );
                    // 保费手头率环比上月
                    double attachingRateChange = MyPOIUtils.getNumbericCellValue( row, 6 );
                    // 当月客户渗透率
                    double customerHandleRateCell = MyPOIUtils.getNumbericCellValue( row, 7 );
                    // 客户渗透率环比上月
                    double customerHandleRateChangeCell = MyPOIUtils.getNumbericCellValue( row, 8 );
                    // 当月车非客均保费
                    double noMotoPremiumPerCustomerCell = MyPOIUtils.getNumbericCellValue( row, 9 );
                    // 客均保费环比上月
                    double noMotoPremiumPerCustomerChangeCell =
                            MyPOIUtils.getNumbericCellValue( row, 10 );

                    BITelsalerAttachingRateRecord record = new BITelsalerAttachingRateRecord(
                            LocalDate.now(), name, motoPremium, nomotoPremium,
                            motoPremiumProPortion, attachingRate, attachingRateChange,
                            customerHandleRateCell, customerHandleRateChangeCell,
                            noMotoPremiumPerCustomerCell, noMotoPremiumPerCustomerChangeCell );

                    records.add( record );
                }
                catch ( IllegalStateException error )
                {
                    error.printStackTrace();
                }
                catch ( NumberFormatException error )
                {
                    error.printStackTrace();
                }
                catch ( NullPointerException error )
                {
                    String message = "第" + String.valueOf( rowIndex ) + "行出现NullPointerException异常";
                    logger.error( message, error );
                }
            }
        }
        finally
        {
            try
            {
                if ( wb != null )
                {
                    wb.close();
                }
            }
            catch ( IOException error )
            {
                error.printStackTrace();
            }
        }

        return records;
    }

    /**
     * 从excel文件读取坐席的车非渗透率数据。
     * @param filePath
     * @param hasCaptionRow
     * @param summaryDate
     * @return
     */
    public static ArrayList<BITelsalerRenewalRateRecord> importBITelsalerRenewalRateFromXlsx(
            String filePath, int sheetIndex, boolean hasCaptionRow, LocalDate summaryDate )
            throws IOException
    {
        ArrayList<BITelsalerRenewalRateRecord> records = new ArrayList<>( 200 );

        Workbook wb = null;
        Sheet sheet = null;

        try
        {
            wb = WorkbookFactory.create( new File( filePath ) );
            sheet = wb.getSheetAt( sheetIndex );
            int rowIndex = 0;

            for ( Row row : sheet )
            {
                rowIndex = row.getRowNum();

                if ( hasCaptionRow == true && rowIndex == 0 )
                {
                    continue;
                }

                try
                {
                    String 责任人 = MyPOIUtils.getStringCellValue( row, 0 );

                    if ( 责任人.isEmpty() == true )
                    {
                        continue;
                    }

                    double 机构目标值 = MyPOIUtils.getNumbericCellValue( row, 1 );
                    double 到期数全月 = MyPOIUtils.getNumbericCellValue( row, 2 );
                    double 序时到期数占比 = MyPOIUtils.getNumbericCellValue( row, 3 );
                    double 个车续保率序时 = MyPOIUtils.getNumbericCellValue( row, 4 );
                    double 个车续保率全月 = MyPOIUtils.getNumbericCellValue( row, 5 );
                    double 环比昨日 = MyPOIUtils.getNumbericCellValue( row, 6 );
                    double 环比上月 = MyPOIUtils.getNumbericCellValue( row, 7 );

                    BITelsalerRenewalRateRecord record = new BITelsalerRenewalRateRecord( 责任人,
                            机构目标值, 到期数全月, 序时到期数占比, 个车续保率序时, 个车续保率全月, 环比昨日, 环比上月 );

                    records.add( record );
                }
                catch ( NullPointerException error )
                {
                    String message = "第" + String.valueOf( rowIndex ) + "行出现NullPointerException异常";

                    logger.error( message, error );
                }
            }
        }
        finally
        {
            try
            {
                if ( wb != null )
                {
                    wb.close();
                }
            }
            catch ( Exception error )
            {
                error.printStackTrace();
            }
        }

        return records;
    }

    public static ArrayList<BIDepartmentArchievementRecord> importBIDepartmentArchievementRecords(
            String filePath, int sheetIndex, boolean hasCaptionRow ) throws IOException
    {
        ArrayList<BIDepartmentArchievementRecord> records = new ArrayList<>( 5 );

        Workbook wb = null;

        try
        {
            wb = WorkbookFactory.create( new File( filePath ) );

            Sheet sheet = wb.getSheetAt( sheetIndex );
            int rowIndex = 0;

            for ( Row row : sheet )
            {
                rowIndex = row.getRowNum();

                if ( hasCaptionRow == true && rowIndex == 0 )
                {
                    continue;
                }

                try
                {
                    String departmentName = MyPOIUtils.getStringCellValue( row, 0 );

                    if ( departmentName.isEmpty() || departmentName.equals( "合计" ) )
                    {
                        continue;
                    }

                    double departmentObject = MyPOIUtils.getNumbericCellValue( row, 1 );
                    double objectGap = MyPOIUtils.getNumbericCellValue( row, 1 );
                    double motoPremium = MyPOIUtils.getNumbericCellValue( row, 1 );
                    double motoPremiumProPortion = MyPOIUtils.getNumbericCellValue( row, 1 );
                    double nomotoPremium = MyPOIUtils.getNumbericCellValue( row, 1 );
                    double attachingRate = MyPOIUtils.getNumbericCellValue( row, 1 );
                    double attachingRateChange = MyPOIUtils.getNumbericCellValue( row, 1 );
                    double customerHandleRate = MyPOIUtils.getNumbericCellValue( row, 1 );
                    double customerHandleRateChange = MyPOIUtils.getNumbericCellValue( row, 1 );
                    double premiumPerCustomer = MyPOIUtils.getNumbericCellValue( row, 1 );
                    double premiumPerCustomerChange = MyPOIUtils.getNumbericCellValue( row, 1 );

                    BIDepartmentArchievementRecord record = new BIDepartmentArchievementRecord(
                            departmentName, departmentObject, objectGap, motoPremium,
                            motoPremiumProPortion, nomotoPremium, attachingRate,
                            attachingRateChange, customerHandleRate, customerHandleRateChange,
                            premiumPerCustomer, premiumPerCustomerChange );

                    records.add( record );
                }
                catch ( NullPointerException error )
                {
                    String message = "第" + String.valueOf( rowIndex ) + "行出现NullPointerException异常";

                    logger.error( message, error );
                }
            }
        }
        finally
        {
            try
            {
                if ( wb != null )
                {
                    wb.close();
                }
            }
            catch ( Exception error )
            {
                String message = "关闭" + filePath + "时出现异常！";

                logger.error( message, error );
            }
        }

        return records;
    }
}
